/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_pca.c
  * @author  sinowealth
  * @version V1.0.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sc0_pca.h"

/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/** @defgroup PCA_MODULE PCA 
  * @brief PCA driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/


/** @defgroup PCA_Public_Functions Public Functions
  * @{
  */ 



/**
  * @brief  Reset the PCAx peripheral registers to their default reset values.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @retval None
  */
void PCA_Reset(PCA_TypeDef* PCAx)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
  if (PCAx == PCA0)
        RCC_AHBPeriphReset(RCC_AHB_PCA0);
	else if (PCAx == PCA1)
        RCC_AHBPeriphReset(RCC_AHB_PCA1);
	else if (PCAx == PCA2)
        RCC_AHBPeriphReset(RCC_AHB_PCA2);
	else if (PCAx == PCA3)
        RCC_AHBPeriphReset(RCC_AHB_PCA3);
}


/**
  * @brief  Enables or disables the PCA's registers protection.
  * @param  PCAx: where x can be 0 or 3 to select the PCAx peripheral.
  * @param  NewState: new state of the PCA's registers protection.
  *          This parameter can be one of the following values:
*            @arg SH_ENABLE:
*            @arg SH_DISABLE:
  * @retval None
  */
void PCA_RegisterWriteLock(PCA_TypeDef* PCAx, SH_FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_FUNCTION_STATE(NewState));

	if (NewState != SH_DISABLE)
	{
			/* Enable PCA registers protection */
			PCAx->LCKR = 0x33CC;
	}
	else
	{
			/* Disable PCA registers protection */
			PCAx->LCKR = 0x0;
	}
}

/**
  * @brief  On or off the specified PCA peripheral.
  * @param  PCAx: where x can be 0 or 3 to select the PCAx peripheral.
  * @param  OnOffState: state of the PCAx peripheral.
  *          This parameter can be one of the following values:
*            @arg SH_ON: PCA on
*            @arg SH_OFF: PCA off
  * @retval None
  */
void PCA_OnOff(PCA_TypeDef* PCAx, SH_CmdState OnOffState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_CMD_STATE(OnOffState));

	PCA_REG_UNLOCK(PCAx);

	if (OnOffState != SH_OFF)
	{
	    PCAx->CR = PCA_CR_PEN_Msk;
	}
	else
	{
	    PCAx->CR = 0;
	} 

	PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Fills each PCA_TimeBaseInitStruct member with its default value.
  * @param  PCA_TimeBaseInitStruct: pointer to a @ref PCA_TimeBaseInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_TimeBaseStructInit(PCA_TimeBaseInitTypeDef* PCA_TimeBaseInitStruct)
{
	PCA_TimeBaseInitStruct->PCAMode = CCMR_SM_FREQ;
	PCA_TimeBaseInitStruct->CounterMode = CFGR_SDEN_UP;
	PCA_TimeBaseInitStruct->InitialLevel = CCMR_TCP_HIGHLEVEL;
	PCA_TimeBaseInitStruct->Prescaler = 0x00;
	PCA_TimeBaseInitStruct->Period = 0x0000FFFF;
	PCA_TimeBaseInitStruct->Count = 0x0;
	PCA_TimeBaseInitStruct->InitA_Data		= 0x8000; 
	PCA_TimeBaseInitStruct->ReloadA_Data	= 0x8000; 
	PCA_TimeBaseInitStruct->InitB_Data		= 0x8000; 
	PCA_TimeBaseInitStruct->ReloadB_Data	= 0x8000; 
	PCA_TimeBaseInitStruct->InitC_Data		= 0x8000; 
	PCA_TimeBaseInitStruct->ReloadC_Data	= 0x8000;

}

/**
  * @brief  Initializes the PCAx Time Base Unit peripheral according to 
  *         the specified parameters in the PCA_TimeBaseInitStruct.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_TimeBaseInitStruct: pointer to a @ref PCA_TimeBaseInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_TimeBaseInit(PCA_TypeDef* PCAx, PCA_TimeBaseInitTypeDef* PCA_TimeBaseInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_TimeBaseInitStruct->PCAMode));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	PCAx->CFGR.V32 |= PCA_TimeBaseInitStruct->CounterMode;
	
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk); 
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk); 
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk); 

	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_TimeBaseInitStruct->PCAMode
										|PCA_TimeBaseInitStruct->InitialLevel
										|CCMR_CEN_EN);
	
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_TimeBaseInitStruct->PCAMode
										|PCA_TimeBaseInitStruct->InitialLevel
										|CCMR_CEN_EN);
	
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR2.V32 |=	 ( PCA_TimeBaseInitStruct->PCAMode
										|PCA_TimeBaseInitStruct->InitialLevel
										|CCMR_CEN_EN);

  /* Set PCA Prescaler ,Period and count register value */
	PCAx->PSC = PCA_TimeBaseInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_TimeBaseInitStruct->Period;
	PCAx->CNT.V32 = PCA_TimeBaseInitStruct->Count;
	
	/* Set initial data and reload data */
	PCAx->CCR0.BIT.CCR0L = PCA_TimeBaseInitStruct->InitA_Data;
	PCAx->CCR0.BIT.CCR0H = PCA_TimeBaseInitStruct->ReloadA_Data;
	PCAx->CCR1.BIT.CCR1L = PCA_TimeBaseInitStruct->InitB_Data;
	PCAx->CCR1.BIT.CCR1H = PCA_TimeBaseInitStruct->ReloadB_Data;
	PCAx->CCR2.BIT.CCR2L = PCA_TimeBaseInitStruct->InitC_Data;
	PCAx->CCR2.BIT.CCR2H = PCA_TimeBaseInitStruct->ReloadC_Data;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Fills each PCA_CaptureInitStruct member with its default value.
  * @param  PCA_CaptureInitStruct : pointer to a @ref PCA_CaptureInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_CaptureStructInit(PCA_CaptureInitTypeDef* PCA_CaptureInitStruct)
{
	PCA_CaptureInitStruct->PCAMode = CCMR_SM_CAPTURE;
	PCA_CaptureInitStruct->Prescaler = 0x00;
	PCA_CaptureInitStruct->Count = 0x0;
	PCA_CaptureInitStruct->CounterMode = CFGR_SDEN_UP;
	PCA_CaptureInitStruct->TriggerLevel = CCMR_FS_UP_DOWN_CAPTURE;
	PCA_CaptureInitStruct->InputFilter = CCMR_ICF_320PCLK;
	PCA_CaptureInitStruct->CaptureClear = CCMR_CC_EN;
	PCA_CaptureInitStruct->CaptureSelect = CCMR_CSSEL_PCAxX;	
	PCA_CaptureInitStruct->Period = 0xFFF;
}


/**
  * @brief  Initializes the PCAx Capture Unit peripheral according to 
  *         the specified parameters in the PCA_CaptureInitStruct.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_CaptureInitStruct: pointer to a @ref PCA_CaptureInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_CaptureInit(PCA_TypeDef* PCAx, PCA_CaptureInitTypeDef* PCA_CaptureInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_CaptureInitStruct->PCAMode));
	assert_param(IS_PCA_CAPTURE_FILTER(PCA_CaptureInitStruct->InputFilter));
	assert_param(IS_PCA_CAPTURE_SELECT(PCA_CaptureInitStruct->CaptureSelect));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	PCAx->CFGR.V32 |= PCA_CaptureInitStruct->CounterMode;
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_ICF_Msk | PCA_CCMR0_SM_Msk | PCA_CCMR0_FS_Msk | PCA_CCMR0_TCP_Msk | PCA_CCMR0_CSSEL_Msk |  PCA_CCMR0_CC_Msk | PCA_CCMR0_CEN_Msk); 
	/* Set the PCA mode, the trigger level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_CaptureInitStruct->PCAMode
										|PCA_CaptureInitStruct->TriggerLevel
										|PCA_CaptureInitStruct->InputFilter
										|PCA_CaptureInitStruct->CaptureClear
										|PCA_CaptureInitStruct->CaptureSelect
										|CCMR_CEN_EN);
	/* Reset the  bits */
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_ICF_Msk | PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk | PCA_CCMR1_CSSEL_Msk| PCA_CCMR1_CC_Msk | PCA_CCMR1_CEN_Msk); 
	/* Set the PCA mode, the trigger level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_CaptureInitStruct->PCAMode
										|PCA_CaptureInitStruct->TriggerLevel
										|PCA_CaptureInitStruct->InputFilter
										|PCA_CaptureInitStruct->CaptureClear
										|PCA_CaptureInitStruct->CaptureSelect
										|CCMR_CEN_EN);
	/* Reset the  bits */
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_ICF_Msk | PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk | PCA_CCMR2_CSSEL_Msk| PCA_CCMR2_CC_Msk | PCA_CCMR2_CEN_Msk); 
	/* Set the PCA mode, the trigger level of output and enable bit */
	PCAx->CCMR2.V32 |=	 ( PCA_CaptureInitStruct->PCAMode
										|PCA_CaptureInitStruct->TriggerLevel
										|PCA_CaptureInitStruct->InputFilter
										|PCA_CaptureInitStruct->CaptureClear
										|PCA_CaptureInitStruct->CaptureSelect
									  |CCMR_CEN_EN);

  /* Set PCA Prescaler and Period register value */
	PCAx->PSC = PCA_CaptureInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_CaptureInitStruct->Period;
	PCAx->CNT.V32 = PCA_CaptureInitStruct->Count;
	
  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Fills each PCA_CMPInitStruct member with its default value.
  * @param  PCA_CMPInitStruct : pointer to a @ref PCA_CMPInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_CMPStructInit(PCA_CMPInitTypeDef* PCA_CMPInitStruct)
{
	PCA_CMPInitStruct->PCAMode = CCMR_SM_CMP;
	PCA_CMPInitStruct->Prescaler = 0x00;
	PCA_CMPInitStruct->Count = 0x00;
	PCA_CMPInitStruct->CounterMode = CFGR_SDEN_UP;
	PCA_CMPInitStruct->CMPMode = CCMR_FS_CONTINUE_CMP;
	PCA_CMPInitStruct->InitialLevel = CCMR_TCP_HIGHLEVEL;
	
	PCA_CMPInitStruct->Period = 0xFF;
	PCA_CMPInitStruct->CMPA_Value = 0x80;
	PCA_CMPInitStruct->CMPB_Value = 0x40;
	PCA_CMPInitStruct->CMPC_Value = 0x20;
}


/**
  * @brief  Initializes the PCAx Compare Unit peripheral according to 
  *         the specified parameters in the PCA_CMPInitStruct.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_CMPInitStruct: pointer to a @ref PCA_CMPInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_CMPInit(PCA_TypeDef* PCAx, PCA_CMPInitTypeDef* PCA_CMPInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_CMPInitStruct->PCAMode));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	PCAx->CFGR.V32 |= PCA_CMPInitStruct->CounterMode;
	
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk | PCA_CCMR0_CEN_Msk); 
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_CMPInitStruct->PCAMode
										|PCA_CMPInitStruct->InitialLevel
										|PCA_CMPInitStruct->CMPMode
										|CCMR_CEN_EN);
	
	/* Reset the  bits */
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk | PCA_CCMR1_CEN_Msk); 
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_CMPInitStruct->PCAMode
										|PCA_CMPInitStruct->InitialLevel
										|PCA_CMPInitStruct->CMPMode
										|CCMR_CEN_EN);
	
	/* Reset the  bits */
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk | PCA_CCMR2_CEN_Msk); 
	/* Set the PCA mode, the initial level of output and enable bit */
	PCAx->CCMR2.V32 |=	 ( PCA_CMPInitStruct->PCAMode
										|PCA_CMPInitStruct->InitialLevel
										|PCA_CMPInitStruct->CMPMode
										|CCMR_CEN_EN);

  /* Set Compare register value */
	PCAx->PSC = PCA_CMPInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_CMPInitStruct->Period;
	PCAx->CNT.V32 = PCA_CMPInitStruct->Count;

	/* Set initial data and reload data */
	PCAx->CCR0.V32 = PCA_CMPInitStruct->CMPA_Value;
	PCAx->CCR1.V32 = PCA_CMPInitStruct->CMPB_Value;
	PCAx->CCR2.V32 = PCA_CMPInitStruct->CMPC_Value; 

  PCA_REG_LOCK(PCAx);
}




/**
  * @brief  Fills each PCA_PWMInitStruct member with its default value.
  * @param  PCA_PWMInitStruct : pointer to a @ref PCA_PWMInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_PWMStructInit(PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */
	PCA_PWMInitStruct->PCAMode = CCMR_SM_PWM;
	PCA_PWMInitStruct->PWMMode = CCMR_FS_PWM3;
	PCA_PWMInitStruct->InitialLevel = CCMR_TCP_DUTYLOW;
	
	/* Set the counter mode*/
	if((PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1) || (PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM2))
		PCA_PWMInitStruct->CounterMode = CFGR_SDEN_UP;
	else
		PCA_PWMInitStruct->CounterMode = CFGR_SDEN_CENTRAL;
	
	PCA_PWMInitStruct->Prescaler = 0x00;
	PCA_PWMInitStruct->Count = 0x00;
		
	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		/* Set initial data and reload data */
		PCA_PWMInitStruct->Period = 0x0000FFFF;
		PCA_PWMInitStruct->InitA_Data		= 0x800; 
		PCA_PWMInitStruct->ReloadA_Data	= 0x800; 
		PCA_PWMInitStruct->InitB_Data		= 0x800; 
		PCA_PWMInitStruct->ReloadB_Data	= 0x800; 
		PCA_PWMInitStruct->InitC_Data		= 0x800; 
		PCA_PWMInitStruct->ReloadC_Data	= 0x800;
	}
	else
	{
	  /* Set PWM Duty data*/
		PCA_PWMInitStruct->Period = 0xFF;
		PCA_PWMInitStruct->DutyA_Value = 0x80;
		PCA_PWMInitStruct->DutyB_Value = 0x40;
		PCA_PWMInitStruct->DutyC_Value = 0x20;
	}
}

/**
  * @brief  Initializes the PCAx Time Base Unit peripheral according to 
  *         the specified parameters in the PCA_TimeBaseInitStruct.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the specified PCA peripheral.
  * @retval None
  */
void PCA_PWMInit(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
  assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
  assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));
	
  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;
	
	/* Reset the  bits */
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk | PCA_CCMR0_CEN_Msk); 
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */
	PCAx->CCMR0.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);

	/* Reset the  bits */
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk | PCA_CCMR1_CEN_Msk); 
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */
	PCAx->CCMR1.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);
										
	/* Reset the  bits */
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk | PCA_CCMR2_CEN_Msk);
	/* Set the PCA mode, the PWM mode, the initial level of output and enable bit */	
	PCAx->CCMR2.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);

  /* Set PCA Prescaler and Period register value */
	PCAx->PSC = PCA_PWMInitStruct->Prescaler;
	PCAx->PR.V32 = PCA_PWMInitStruct->Period;
	PCAx->CNT.V32 = PCA_PWMInitStruct->Count;

	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->PR.V32 = 0x0000FFFF;
		PCAx->CCR0.BIT.CCR0L = PCA_PWMInitStruct->InitA_Data;
		PCAx->CCR0.BIT.CCR0H = PCA_PWMInitStruct->ReloadA_Data;
		PCAx->CCR1.BIT.CCR1L = PCA_PWMInitStruct->InitB_Data;
		PCAx->CCR1.BIT.CCR1H = PCA_PWMInitStruct->ReloadB_Data;
		PCAx->CCR2.BIT.CCR2L = PCA_PWMInitStruct->InitC_Data;
		PCAx->CCR2.BIT.CCR2H = PCA_PWMInitStruct->ReloadC_Data;
	}
	else
	{
		PCAx->PR.V32 = PCA_PWMInitStruct->Period;
		PCAx->CCR0.V32 = PCA_PWMInitStruct->DutyA_Value;
		PCAx->CCR1.V32 = PCA_PWMInitStruct->DutyB_Value;
		PCAx->CCR2.V32 = PCA_PWMInitStruct->DutyC_Value; 
	}
	
  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Initialize the PCAxA function according to 
  *         the specified parameters in the PCA_PWMInitStruct.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the PWM0 function.
  * @retval None
  */
void PCA_PCAxAPWMConfig(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
  assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
  assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;

 	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->CCR0.BIT.CCR0L = PCA_PWMInitStruct->InitA_Data;
		PCAx->CCR0.BIT.CCR0H = PCA_PWMInitStruct->ReloadA_Data;
	}
	else
	{
		PCAx->CCR0.V32 = PCA_PWMInitStruct->DutyA_Value;
	}
	PCAx->CCMR0.V32 &= ~( PCA_CCMR0_SM_Msk| PCA_CCMR0_FS_Msk| PCA_CCMR0_TCP_Msk); 
	PCAx->CCMR0.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);	
  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Initialize the PCAxB function according to 
  *         the specified parameters in the PCA_PWMInitStruct.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the PWM1 function.
  * @retval None
  */
void PCA_PCAxBPWMConfig(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
  assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
  assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));

  PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;
	
	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->CCR1.BIT.CCR1L = PCA_PWMInitStruct->InitB_Data;
		PCAx->CCR1.BIT.CCR1H = PCA_PWMInitStruct->ReloadB_Data;
	}
	else
	{
		PCAx->CCR1.V32 = PCA_PWMInitStruct->DutyB_Value;
	}
	PCAx->CCMR1.V32 &= ~( PCA_CCMR1_SM_Msk| PCA_CCMR1_FS_Msk| PCA_CCMR1_TCP_Msk); 
	PCAx->CCMR1.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);	

  PCA_REG_LOCK(PCAx);

}



/**
  * @brief  Initialize the PCAxC function according to 
  *         the specified parameters in the PCA_PWMInitStruct.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_PWMInitStruct: pointer to a @ref PCA_PWMInitTypeDef structure
  *         that contains the configuration information for the PWM2 function.
  * @retval None
  */
void PCA_PCAxCPWMConfig(PCA_TypeDef* PCAx, PCA_PWMInitTypeDef* PCA_PWMInitStruct)
{
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_MODE(PCA_PWMInitStruct->PCAMode));
	assert_param(IS_PCA_PWM_MODE(PCA_PWMInitStruct->PWMMode));
	assert_param(IS_PCA_INIT_LEVEL(PCA_PWMInitStruct->InitialLevel));

	PCA_REG_UNLOCK(PCAx);
	/* Reset the  bits */
	PCAx->CFGR.V32 &= ~PCA_CFGR_SDEN_Msk; 
	/* Set the counter mode*/
	PCAx->CFGR.V32 |= PCA_PWMInitStruct->CounterMode;
	
	if(PCA_PWMInitStruct->PWMMode == CCMR_FS_PWM1)
	{
		PCAx->CCR2.BIT.CCR2L = PCA_PWMInitStruct->InitC_Data;
		PCAx->CCR2.BIT.CCR2H = PCA_PWMInitStruct->ReloadC_Data;
	}
	else
	{
		PCAx->CCR2.V32 = PCA_PWMInitStruct->DutyC_Value;
	}
	
	PCAx->CCMR2.V32 &= ~( PCA_CCMR2_SM_Msk| PCA_CCMR2_FS_Msk| PCA_CCMR2_TCP_Msk); 
	PCAx->CCMR2.V32 |=	 ( PCA_PWMInitStruct->PCAMode
										|PCA_PWMInitStruct->PWMMode 
										|PCA_PWMInitStruct->InitialLevel
										|CCMR_CEN_EN);	
	
  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Sets the PCAx Counter Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  Counter: specifies the Counter register new value.
  * @retval None
  */
void PCA_SetCounter(PCA_TypeDef* PCAx, uint32_t Counter)
{
	/* Check the parameters */
  assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CNT.V32 = Counter;	

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Sets the PCAx Prescaler Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  Prescaler: specifies the Prescaler register new value.
  * @retval None
  */
void PCA_SetPrescaler(PCA_TypeDef* PCAx, uint32_t Prescaler)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	PCA_REG_UNLOCK(PCAx);

	PCAx->PSC = Prescaler;	

	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Sets the PCAx Period Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  Period: specifies the Period register new value.
  * @retval None
  */
void PCA_SetPeriod(PCA_TypeDef* PCAx, uint32_t Period)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	PCA_REG_UNLOCK(PCAx);

	PCAx->PR.V32 = Period;
	
	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Get the PCAx Counter Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @retval The Counter Register value.
  */
uint32_t PCA_GetCounter(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CNT.V32;	
}

/**
  * @brief  Get the PCAx Prescaler Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @retval The Prescaler Register value.
  */
uint32_t PCA_GetPrescaler(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->PSC;	
}

/**
  * @brief  Get the PCAx Period Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @retval The Period Register value.
  */
uint32_t PCA_GetPeriod(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->PR.V32;	
}


/**
  * @brief  Set the PCAxA Duty Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  Duty: specifies the PCAxA Duty Register new value.
  * @retval None
  */
void PCA_SetCCR0(PCA_TypeDef* PCAx, uint32_t Duty)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	PCA_REG_UNLOCK(PCAx);

	PCAx->CCR0.V32 = Duty;

	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the PCAxB Duty Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  Duty: specifies the PCAxB Duty Register new value.
  * @retval None
  */
void PCA_SetCCR1(PCA_TypeDef* PCAx, uint32_t Duty)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	PCA_REG_UNLOCK(PCAx);

	PCAx->CCR1.V32 = Duty;

	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the PCAxC Duty Register value
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  Duty: specifies the PCAxC Duty Register new value.
  * @retval None
  */
void PCA_SetCCR2(PCA_TypeDef* PCAx, uint32_t Duty)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	PCA_REG_UNLOCK(PCAx);

	PCAx->CCR2.V32 = Duty;

	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Get the PCAxA Duty Register value.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @retval The PCAxA Duty Register value.
  */
uint32_t PCA_GetCCR0(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CCR0.V32;
}

/**
  * @brief  Set the PCAxB Duty Register value.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @retval The PCAxB Duty Register value.
  */
uint32_t PCA_GetCCR1(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CCR1.V32;
}

/**
  * @brief  Get the PCAxC Duty Register value.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @retval The PCAxC Duty Register value.
  */
uint32_t PCA_GetCCR2(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	return (uint32_t)PCAx->CCR2.V32;
}



/**
  * @brief  Enable or Disable specified PCA interrupts.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  PCA_INT: specifie the PCA interrupts sources to be enabled or disabled.
  *          This parameter can be any combination of following values:
*            @arg PCA_INT_CIE: PCAx Counter/timer overflow interrupt enable
*            @arg PCA_INT_PIE: PCAx Counter/timer period interrupt enable
*            @arg PCA_INT_CCIE0: PCAxA Capture/Compare Interrupt Control
*            @arg PCA_INT_CCIE1: PCAxB Capture/Compare Interrupt Control
*            @arg PCA_INT_CCIE2: PCAxC Capture/Compare Interrupt Control

  * @param  NewState: new state of interrupt sources.
  *          This parameter can be one of the following values:
*            @arg SH_ENABLE: Be enable
*            @arg SH_DISABLE: Be disable
  * @retval None
  */
void PCA_INTConfig(PCA_TypeDef* PCAx, uint32_t PCA_INT, SH_FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_INT(PCA_INT));
	assert_param(IS_FUNCTION_STATE(NewState));  

	PCA_REG_UNLOCK(PCAx);

	if (NewState != SH_DISABLE)
	{
			/* Enable the interrupt sources */
			if ((PCA_INT & PCA_INT_CIE) == PCA_INT_CIE)
			{
					PCAx->CFGR.V32 |= CFGR_CIE_EN;
			}
			if ((PCA_INT & PCA_INT_PIE) == PCA_INT_PIE)
			{
					PCAx->CFGR.V32 |= CFGR_PIE_EN;
			}	
			if ((PCA_INT & PCA_INT_CCIE0) == PCA_INT_CCIE0)
			{
					PCAx->CCMR0.V32 |= CCMR_CCIE_EN;
			}	
			if ((PCA_INT & PCA_INT_CCIE1) == PCA_INT_CCIE1)
			{
					PCAx->CCMR1.V32 |= CCMR_CCIE_EN;
			}		
			if ((PCA_INT & PCA_INT_CCIE2) == PCA_INT_CCIE2)
			{
					PCAx->CCMR2.V32 |= CCMR_CCIE_EN;
			}						
	}
	else
	{
			/* Disable the interrupt sources */
			if ((PCA_INT & PCA_INT_CIE) == PCA_INT_CIE)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CIE_Msk);
			}
			if ((PCA_INT & PCA_INT_PIE) == PCA_INT_PIE)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_PIE_Msk);
			}	
			if ((PCA_INT & PCA_INT_CCIE0) == PCA_INT_CCIE0)
			{
					PCAx->CCMR0.V32 &= ~(PCA_CCMR0_CCIE_Msk);
			}	
			if ((PCA_INT & PCA_INT_CCIE1) == PCA_INT_CCIE1)
			{
					PCAx->CCMR1.V32 &= ~(PCA_CCMR1_CCIE_Msk);
			}		
			if ((PCA_INT & PCA_INT_CCIE2) == PCA_INT_CCIE2)
			{
					PCAx->CCMR2.V32 &= ~(PCA_CCMR2_CCIE_Msk);
			}		
	}
	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Enable or Disable specified PCA interrupts.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  PCA_INT: specifie the PCA interrupts sources to be enabled or disabled.
  *          This parameter can be any combination of following values:
*            @arg PCA_INT_CIE: PCAx Counter/timer overflow interrupt enable
*            @arg PCA_INT_PIE: PCAx Counter/timer period interrupt enable
*            @arg PCA_INT_CCIE0: PCAxA Capture/Compare Interrupt Control
*            @arg PCA_INT_CCIE1: PCAxB Capture/Compare Interrupt Control
*            @arg PCA_INT_CCIE2: PCAxC Capture/Compare Interrupt Control

  * @param  NewState: new state of interrupt sources.
  *          This parameter can be one of the following values:
*            @arg SH_ENABLE: Be enable
*            @arg SH_DISABLE: Be disable
  * @retval None
  */
void PCA_DMAConfig(PCA_TypeDef* PCAx, uint32_t PCA_DMA, SH_FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_INT(PCA_DMA));
	assert_param(IS_FUNCTION_STATE(NewState));  

	PCA_REG_UNLOCK(PCAx);

	if (NewState != SH_DISABLE)
	{
			/* Enable the interrupt sources */
			if ((PCA_DMA & PCA_DMA_CU) == PCA_DMA_CU)
			{
					PCAx->CFGR.V32 |= (PCA_DMA_CU<<20);
			}
			if ((PCA_DMA & PCA_DMA_PU) == PCA_DMA_PU)
			{
					PCAx->CFGR.V32 |= (PCA_DMA_PU<<20);
			}	
			if ((PCA_DMA & PCA_DMA_CC0) == PCA_DMA_CC0)
			{
					PCAx->CFGR.V32 |= (PCA_DMA_CC0<<20);
			}	
			if ((PCA_DMA & PCA_DMA_CC1) == PCA_DMA_CC1)
			{
					PCAx->CFGR.V32 |= (PCA_DMA_CC1<<20);
			}		
			if ((PCA_DMA & PCA_DMA_CC2) == PCA_DMA_CC2)
			{
					PCAx->CFGR.V32 |= (PCA_DMA_CC2<<20);
			}						
	}
	else
	{
			/* Disable the interrupt sources */
			if ((PCA_DMA & PCA_DMA_CU) == PCA_DMA_CU)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CUDMA_Msk);
			}
			if ((PCA_DMA & PCA_DMA_PU) == PCA_DMA_PU)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_PUDMA_Msk);
			}	
			if ((PCA_DMA & PCA_DMA_CC2) == PCA_DMA_CC0)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CC0DMA_Msk);
			}	
			if ((PCA_DMA & PCA_DMA_CC2) == PCA_DMA_CC1)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CC1DMA_Msk);
			}		
			if ((PCA_DMA & PCA_DMA_CC2) == PCA_DMA_CC2)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CC2DMA_Msk);
			}		
	}
	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the specified of PCA update moment.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  PCA_Update: specifie the update moment of PACx period/CCR2/CCR1/CCR0 to be enabled or disabled.
  *          This parameter can be any combination of following values:
*            @arg PCA_PUE:PACx period register update control bit
*            @arg PCA_CC0UE:PACxA Module update control bit
*            @arg PCA_CC1UE:PACxB Module update control bit
*            @arg PCA_CC2UE:PACxC Module update control bit

  * @param  NewState: new state of the update moment.
  *          This parameter can be one of the following values:
*            @arg PCA_UPDATE_NEXT_CIRCLE: be valid in the next cycle
*            @arg PCA_UPDATE_NOW: be valid immediately at the moment
  * @retval None
  */
void PCA_UpdateConfig(PCA_TypeDef* PCAx, uint32_t PCA_Update, SH_FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_UPDATE(PCA_Update));

	PCA_REG_UNLOCK(PCAx);

	if (NewState != PCA_UPDATE_NEXT_CIRCLE)
	{
			if ((PCA_Update & PCA_PUE) == PCA_PUE)
			{
					PCAx->CFGR.V32 |= CFGR_PUE_EN;
			}
			if ((PCA_Update & PCA_CC0UE) == PCA_CC0UE)
			{
					PCAx->CFGR.V32 |= CFGR_CC0UE_EN;
			}	
			if ((PCA_Update & PCA_CC1UE) == PCA_CC1UE)
			{
					PCAx->CFGR.V32 |= CFGR_CC1UE_EN;
			}	
			if ((PCA_Update & PCA_CC2UE) == PCA_CC2UE)
			{
					PCAx->CFGR.V32 |= CFGR_CC2UE_EN;
			}		
			
	}
	else
	{
			if ((PCA_Update & PCA_PUE) == PCA_PUE)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_PUE_Msk);
			}
			if ((PCA_Update & PCA_CC0UE) == PCA_CC0UE)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CC0UE_Msk);
			}	
			if ((PCA_Update & PCA_CC1UE) == PCA_CC1UE)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CC1UE_Msk);
			}	
			if ((PCA_Update & PCA_CC2UE) == PCA_CC2UE)
			{
					PCAx->CFGR.V32 &= ~(PCA_CFGR_CC2UE_Msk);
			}		

	}
	PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Check whether the specified PCA flag is set or not
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  PCA_Flag: specifie the flag to check.
  *          This parameter can be one of the following values:
  *            @arg PCA_FLAG_CC0IF:PCAxA Capture/Compare Flag
  *            @arg PCA_FLAG_CC1IF:PCAxB Capture/Compare Flag
  *            @arg PCA_FLAG_CC2IF:PCAxC Capture/Compare Flag
  *            @arg PCA_FLAG_FLT1IF:Fault1 Detection Interrupt Flag
  *            @arg PCA_FLAG_FLT2IF:Fault2 Detection Interrupt Flag
  *            @arg PCA_FLAG_PIF:PCAx time base periodic matching interrupt flag bit (only valid in center aligned mode)
  *            @arg PCA_FLAG_CIF:PCA counter overflow interrupt flag bit

  * @retval None
  */
SH_FlagStatus PCA_GetFlagStatus(PCA_TypeDef* PCAx, uint32_t PCA_Flag)
{
  SH_FlagStatus bitStatus;
    
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_GET_ONE_FLAG(PCA_Flag));

	if ((PCAx->SR.V32 & PCA_Flag) != (uint32_t)SH_RESET)
	{
			bitStatus = SH_SET;
	}
	else
	{
			bitStatus = SH_RESET;
	}
	
	return bitStatus;
}

/**
  * @brief  Clear the PCAx's pending flags.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_Flag: specifie the flag bit to clear.
  *          This parameter can be any combination of the following values:
*            @arg PCA_FLAG_CC0IF:PCAxA Capture/Compare Flag
*            @arg PCA_FLAG_CC1IF:PCAxB Capture/Compare Flag
*            @arg PCA_FLAG_CC2IF:PCAxC Capture/Compare Flag
*            @arg PCA_FLAG_FLT1IF:Fault Detection Interrupt Flag
*            @arg PCA_FLAG_FLT2IF:Fault Detection Interrupt Flag
*            @arg PCA_FLAG_PIF:PCAx time base periodic matching interrupt flag bit (only valid in center aligned mode)
*            @arg PCA_FLAG_CIF:PCAx counter overflow interrupt flag bit

  * @retval None
  */
void PCA_ClearFlag(PCA_TypeDef* PCAx, uint32_t PCA_Flag)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_GET_ONE_FLAG(PCA_Flag));

	PCA_REG_UNLOCK(PCAx);
	PCAx->SR.V32 = (PCA_Flag<<16);
	PCA_REG_LOCK(PCAx);

}


/**
  * @brief  Enable force manual output mode of the specified PWM port.
  * @param  PCAx: Where x can be 0 or 3 to select the PCA peripheral.
  * @param  PCAxPin: specifie PWM port to be enabled or disabled.
  *          This parameter can be any combination of following values:
*            @arg PCAxA_FORCE:PCAxA force output setting bit
*            @arg PCAxB_FORCE:PCAxB force output setting bit
*            @arg PCAxC_FORCE:PCAxC force output setting bit
  * @retval None
  */
void PCA_SetForceOut(PCA_TypeDef* PCAx, uint8_t PCAxPin)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_FORCE_PIN(PCAxPin));

	PCA_REG_UNLOCK(PCAx);

	/* Enable manual output mode of specified PCA port */
	PCAx->FORCE.V32 |= (1<<PCAxPin); 

	PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Set the PCAx Trigger ADC Register value.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  Value: specifie Trigger ADC Register value.
  *          This parameter can be 0 to 0xFFFFFFFF.
  * @retval None
  */
void PCA_SetADCTriggerValue(PCA_TypeDef* PCAx, uint32_t Value)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->ADTR = Value;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the PCAx clock source.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  Source: specifie the clock source.
  *            @arg CFGR_CPS_PCLK: PCLK
  *            @arg CFGR_CPS_ECI: PCAxECI Falling edge
  *            @arg CFGR_CPS_RC128K: RC128K
  * @retval None
  */
void PCA_SetPCAClock(PCA_TypeDef* PCAx, uint32_t Source)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CFGR.V32 &= ~(PCA_CFGR_CPS_Msk);
	PCAx->CFGR.V32 |= Source;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the filter time, when the PCAx clock source is PCAxECI.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  Filter: specifie the clock source.
*            @arg CFGR_ECF_NONE: No filter
*            @arg CFGR_ECF_8PCLK: Filter time constant is 8*PCLK
*            @arg CFGR_ECF_16PCLK: Filter time constant is 16*PCLK
*            @arg CFGR_ECF_32PCLK: Filter time constant is 32*PCLK
  * @retval None
  */
void PCA_SetPCAxECIFilter(PCA_TypeDef* PCAx, uint32_t Filter)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_ECI_FILTER(Filter));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CFGR.V32 &= ~(PCA_CFGR_ECF_Msk);
	PCAx->CFGR.V32 |= Filter;

  PCA_REG_LOCK(PCAx);
}

/**
  * @brief  Set the synchronization mode of PCAx.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  SYN_Source: specifie the synchronization source.
*            @arg CFGR_SYN_NONE: No synchronization
*            @arg CFGR_SYN_PCA: Be set or cleared by the other PCA synchronously.
*            @arg CFGR_SYN_MCM0: Be set or cleared by MCM synchronously.
*            @arg CFGR_SYN_MCM1: Be set or cleared by MCM synchronously.
  * @retval None
  */
void PCA_SetSYNSource(PCA_TypeDef* PCAx, uint32_t SYN_Source)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	PCAx->CFGR.V32 &= ~(PCA_CFGR_SYN_Msk);
	PCAx->CFGR.V32 |= SYN_Source;

  PCA_REG_LOCK(PCAx);
}


/**
  * @brief  Configure the PCAx ADC trigger timing.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_ADCTrigger: PCAx trigger ADC selection
*            @arg PCA_ADC_TRIGGER_None:No trigger
*            @arg PCA_ADC_TRIGGER_Up: Trigger ADC when counter is incremented
*            @arg PCA_ADC_TRIGGER_Down: Trigger ADC when counter is decremented
*            @arg PCA_ADC_TRIGGER_UpandDown: Trigger ADC when counter is incremented or decremented
  * @retval None
  */
void PCA_ADCTriggerConfig(PCA_TypeDef* PCAx, uint32_t PCA_ADCTrigger)
{
	uint32_t temp = 0;
	
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_REG_UNLOCK(PCAx);

	temp = PCAx->CFGR.V32;
	
	temp &= ~(PCA_CFGR_ADTSEL_Msk);
	temp |= PCA_ADCTrigger;
	
	PCAx->CFGR.V32 = temp;

  PCA_REG_LOCK(PCAx);
}




/**
  * @brief  Fills each PCA_FLTInitStruct member with its default value.
  * @param  PCA_FLTInitStruct : pointer to a @ref PCA_FLTInitTypeDef
  *         structure which will be initialized.
  * @retval None
  */
void PCA_FLTStructInit(PCA_FLTInitTypeDef* PCA_FLTInitStruct)
{
    
  PCA_FLTInitStruct->V32 = 0;    
}


/**
  * @brief  Initialize the PCAx Fault module according to 
  *         the specified parameters in the PCA_FLTInitStruct.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_FLTInitStruct: pointer to a @ref PCA_FLTInitTypeDef structure
  *         that contains the configuration information for the Fault module.
  * @retval None
  */
void PCA_FLTInit(PCA_TypeDef* PCAx, PCA_FLTInitTypeDef* PCA_FLTInitStruct)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_FLT_REG_UNLOCK(PCAx);

	PCAx->FLTCR.V32 = PCA_FLTInitStruct->V32;

  PCA_FLT_REG_LOCK(PCAx);
}


/**
  * @brief  Enable or disable FLT module registers write protection.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  NewState: new state of FLT write protection.
  *          This parameter can be one of the following values:
*            @arg SH_ENABLE: Enable the FLT register write protection
*            @arg SH_DISABLE: Disable the FLT register write protection
  * @retval None
  */
void PCA_FLTWriteLock(PCA_TypeDef* PCAx, SH_FunctionalState NewState)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_FUNCTION_STATE(NewState));
	
	if (NewState != SH_DISABLE)
	{
        /* Enable the FLT register write protection */
		PCAx->FLTWPR = 0x33CC;
	}
	else
	{
        /* Disable the FLT register write protection */
		PCAx->FLTWPR = 0x0000;
	}
}


/**
  * @brief  Configure the PCAx FLT mode and output pin status.  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_FLT_STATE_Mode: specifie the FLT detect mode to set.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTM_LOCK_MODE: Latch mode
*            @arg FLTCR_FLTM_TIME_MODE: Sequential mode
  * @param  PCA_FLT_Int: specifie the control bit of the FLT Interrupt.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTIE_DIS: Fault detection interrupt disables.
*            @arg FLTCR_FLTIE_EN:  Fault detection interrupt enables.
  * @param  PCA_FLT_OutputLevel: specifie the output status of the FLT pin.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FOUT_HIZ:  Outputs high resistance state
*            @arg FLTCR_FOUT_LOW:  Outputs low level
*            @arg FLTCR_FOUT_HIGH: Outputs high level

  * @retval None
  */


/**
  * @brief  Configure input source of the PCAx FLT1 module.   
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_FLTSource: specifie the FLT1 input sources.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTSEL_NO:   CMPx is not used as input source of PCA fault detection 1
*            @arg FLTCR_FLTSEL_LOW:  CMPx is used as input source of PCA fault detection 1, low level valid
*            @arg FLTCR_FLTSEL_HIGH: CMPx is used as input source of PCA fault detection 1, high level valid
  * @param  PCA_FLT_STATE_Mode: specifie the FLT detect mode to set.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTM_LOCK_MODE: Latch mode
*            @arg FLTCR_FLTM_TIME_MODE: Sequential mode
  * @param  PCA_FLT_Int: specifie the control bit of the FLT Interrupt.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FLTIE_DIS: Fault detection interrupt disables.
*            @arg FLTCR_FLTIE_EN:  Fault detection interrupt enables.
  * @param  PCA_FLT_OutputLevel: specifie the output status of the FLT pin.
  *          This parameter can be one of the following values:
*            @arg FLTCR_FOUT_HIZ:  Outputs high resistance state
*            @arg FLTCR_FOUT_LOW:  Outputs low level
*            @arg FLTCR_FOUT_HIGH: Outputs high level
  * @retval None
  */
void PCA_FLT1Config(PCA_TypeDef* PCAx, PCA_FLTInitTypeDef fltcfg)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_FLT_OUTPUT_LEVEL(fltcfg.BITS.FLT_FOut));
	assert_param(IS_PCA_FLT_STATE_MODE(fltcfg.BITS.FLT_F1Mode));
	assert_param(IS_PCA_FLT1_SOURCE(fltcfg.BITS.FLT_CMP0Sel));
	assert_param(IS_PCA_FLT1_SOURCE(fltcfg.BITS.FLT_CMP1Sel));
	assert_param(IS_PCA_FLT1_SOURCE(fltcfg.BITS.FLT_CMP2Sel));

	PCA_FLT_REG_UNLOCK(PCAx);
  PCAx->FLTCR.BIT.FOUT   = fltcfg.BITS.FLT_FOut;
	PCAx->FLTCR.BIT.FLT1IE = fltcfg.BITS.FLT_F1IE;
	PCAx->FLTCR.BIT.FLT1M = fltcfg.BITS.FLT_F1Mode;
	PCAx->FLTCR.BIT.CMP0S = fltcfg.BITS.FLT_CMP0Sel;
	PCAx->FLTCR.BIT.CMP1S = fltcfg.BITS.FLT_CMP1Sel;
	PCAx->FLTCR.BIT.CMP2S = fltcfg.BITS.FLT_CMP2Sel;
    
  PCA_FLT_REG_LOCK(PCAx);

}


/**
  * @brief  Configure filter active pin and active level of the PCAx FLT2 module.   
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  PCA_FLT2Filter: specifie the FLT2 filter value.
  *          This parameter can be one of the following values:    
*            @arg FLTCR_FLT2DEB_0US:	filter time = 0us
*            @arg FLTCR_FLT2DEB_0P5US:filter time = 0.5us	
*            @arg FLTCR_FLT2DEB_1US:	filter time = 1us	
*            @arg FLTCR_FLT2DEB_1P5US:filter time = 1.5us	
*            @arg FLTCR_FLT2DEB_2US:	filter time = 2us	
*            @arg FLTCR_FLT2DEB_3US:	filter time = 3us
*            @arg FLTCR_FLT2DEB_4US:	filter time = 4us		
*            @arg FLTCR_FLT2DEB_6US:	filter time = 6us		
  * @param  PCA_FLT2Mode: Detection function mode selection.
  *          This parameter can be one of the following values:
  *            @arg FLTCR_FLTM_LOCK_MODE: Latch mode
  *            @arg FLTCR_FLTM_TIME_MODE: Sequential mode
  * @param  PCA_FLT_Int: PCA_Fault_Interrupt.
  *          This parameter can be one of the following values:
  *            @arg FLTCR_FLTIE_DIS: Fault detection interrupt control disable
  *            @arg FLTCR_FLTIE_EN: Fault detection interrupt control enable
	
  *            @arg FLTCR_FLTSEL_NO:	Fault detection 2 FLT input is invalid
  *            @arg FLTCR_FLTSEL_LOW:Fault detection 2 FLT input is valid on low level	
  *            @arg FLTCR_FLTSEL_HIGH:	Fault detection 2 FLT input is valid on high level
  * @retval None
  */
void PCA_FLT2Config(PCA_TypeDef* PCAx, PCA_FLTInitTypeDef fltcfg)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_PCA_FLT_OUTPUT_LEVEL(fltcfg.BITS.FLT_FOut));	
	assert_param(IS_PCA_FLT_STATE_MODE(fltcfg.BITS.FLT_F2Mode));
	assert_param(IS_PCA_FLT2_ACTIVE_LEVEL(fltcfg.BITS.FLT_FPin0Sel));
	assert_param(IS_PCA_FLT2_ACTIVE_LEVEL(fltcfg.BITS.FLT_FPin1Sel));
	assert_param(IS_PCA_FLT2_ACTIVE_LEVEL(fltcfg.BITS.FLT_FPin2Sel));
	assert_param(IS_PCA_FLT2_ACTIVE_LEVEL(fltcfg.BITS.FLT_FPin3Sel));

	PCA_FLT_REG_UNLOCK(PCAx);
  PCAx->FLTCR.BIT.FOUT    = fltcfg.BITS.FLT_FOut;
	PCAx->FLTCR.BIT.FLT2DEB = fltcfg.BITS.FLT_F2DEB;
	PCAx->FLTCR.BIT.FLT2IE =  fltcfg.BITS.FLT_F2IE;
	PCAx->FLTCR.BIT.FLT2M =   fltcfg.BITS.FLT_F2Mode;
  PCAx->FLTCR.BIT.FPIN0S =  fltcfg.BITS.FLT_FPin0Sel;
  PCAx->FLTCR.BIT.FPIN1S =  fltcfg.BITS.FLT_FPin1Sel;
  PCAx->FLTCR.BIT.FPIN2S =  fltcfg.BITS.FLT_FPin2Sel;
  PCAx->FLTCR.BIT.FPIN3S =  fltcfg.BITS.FLT_FPin3Sel;

	PCA_FLT_REG_LOCK(PCAx);
}

/**
  * @brief  Set the PCAx's Soft FLT.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @retval None
  */
void PCA_SetSoftFLT(PCA_TypeDef* PCAx)
{
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	PCA_FLT_REG_UNLOCK(PCAx);
  PCAx->SOFTFLT    = 0xAA;
	PCA_FLT_REG_LOCK(PCAx);
}

/**
  * @brief  Clear the PCAx's FLT pending flag.
  * @note   This function only can be called in Latch mode of FLT,because this bit will be
  *         set by hardware in Each Mode.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  FltIdx: PCA_FLT_Mode.
  *          This parameter can be one of the following values:
  *            @arg PCA_FLT0: PCAx Fault 0 
  *            @arg PCA_FLT1: PCAx Fault 1  
  *            @arg PCA_FLT2: PCAx Fault 2 
  * @retval None
  */
void PCA_ClearFLTStatusFlag(PCA_TypeDef* PCAx, int FltIdx)
{
   uint32_t fltmask=0;

	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

  PCA_FLT_REG_UNLOCK(PCAx);
    
	if(FltIdx == PCA_FLT0)
	{
			fltmask = PCA_FLTCR_FLT0STATC_Msk;
	}
	else if(FltIdx  == PCA_FLT1)
	{
			fltmask = PCA_FLTCR_FLT1STATC_Msk;
	}
	else if(FltIdx == PCA_FLT2)
	{
			fltmask = PCA_FLTCR_FLT2STATC_Msk;
	}
   PCAx->FLTCR.V32 |= fltmask;

   PCA_FLT_REG_LOCK(PCAx);
}

/**
  * @brief  Check whether the specified PCA FLT status flag is set or not  
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  FltIdx: PCA_FLT_Mode.
  *          This parameter can be one of the following values:
  *            @arg PCA_FLT0: PCAx Fault 0 
  *            @arg PCA_FLT1: PCAx Fault 1  
  *            @arg PCA_FLT2: PCAx Fault 2 
* @retval SH_FlagStatus: Fault Flag
  */
SH_FlagStatus PCA_GetFLTStatus(PCA_TypeDef* PCAx, int FltIdx)
{
	SH_FlagStatus bitStatus;
  uint32_t fltmask=0;
	
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));

	if(FltIdx == PCA_FLT0)
	{
			fltmask = PCA_FLTCR_FLT0STAT_Msk;
	}
	else if(FltIdx  == PCA_FLT1)
	{
			fltmask = PCA_FLTCR_FLT1STAT_Msk;
	}
	else if(FltIdx == PCA_FLT2)
	{
			fltmask = PCA_FLTCR_FLT2STAT_Msk;
	}
    
	if ((PCAx->FLTCR.V32 & fltmask) != SH_RESET)
	{
		bitStatus = SH_SET;
	}
	else
	{
		bitStatus = SH_RESET;
	}
	
	return bitStatus;
}



/**
  * @brief  Enable or disable the PCAx's Fault module.
  * @param  PCAx: where x can be  0 or 3 to select the PCA peripheral.
  * @param  FLTx: specifie which FLTx module to be selected.
  *          This parameter can be any combination of the following values:
*            @arg PCA_FLT1: Fault detection 1 function
*            @arg PCA_FLT2: Fault detection 2 function
  * @param  OnOffState: state of the fault module.
  *          This parameter can be one of the following values:
*            @arg SH_ON: Enable the function
*            @arg SH_OFF: Disable the function
  * @retval None
  */
void PCA_FLTOnOff(PCA_TypeDef* PCAx, uint32_t FLTx, SH_CmdState OnOffState)
{    
	/* Check the parameters */
	assert_param(IS_PCA_ALL_PERIPH(PCAx));
	assert_param(IS_CMD_STATE(OnOffState));  

  PCA_FLT_REG_UNLOCK(PCAx);

	if (OnOffState != SH_OFF)
	{
		if ((FLTx & PCA_FLT1) == PCA_FLT1)
		{
			PCAx->FLTCR.V32 |= PCA_FLTCR_FLT1EN_Msk;
		}
		if ((FLTx & PCA_FLT2) == PCA_FLT2)
		{
			PCAx->FLTCR.V32 |= PCA_FLTCR_FLT2EN_Msk;
		}
	}
	else
	{
		if ((FLTx & PCA_FLT1) == PCA_FLT1)
		{
			PCAx->FLTCR.V32 &= ~PCA_FLTCR_FLT1EN_Msk;
		}
		if ((FLTx & PCA_FLT2) == PCA_FLT2)
		{
			PCAx->FLTCR.V32 &= ~PCA_FLTCR_FLT2EN_Msk;
		}
	}

  PCA_FLT_REG_LOCK(PCAx);
}


/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/

