/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_rcc.c
  * @author  sinowealth
  * @version V1.0.0
  * @date    07-March-2022 
  * @brief   This file provides reset and clock module's APIs
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *          
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sc0_rcc.h"
#ifndef HSI_CLK
#define HSI_CLK 12000000
#endif
#ifndef HSE_CLK
#define HSE_CLK 8000000
#endif
      
/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/*  reset and clock control module-----------------------------------------------------------*/
/** @defgroup RCC_MODULE  RCC 
   *  reset and clock control module(RCC)
  * @{
  */ 
        
/** @defgroup  RCC_Group_Pub_Funcs  Public Functions
 *  @brief   RCC public functions
 *
  * @{
  */
/**
  *@code   Example    
  *     RCC_ClocksTypeDef rccClocks;
  *     RCC_GetClocksFreq(&rccClocks);
   *@endcode
   */
/**
  * @brief  get current system's clock frequency
  * @param  Clocks: output clock frequency
  * @retval None
  */
void RCC_GetClocksFreq(RCC_Clocks_TypeDef* Clocks)
{
    uint32_t sysClk=0;
    if(RCC->CR.BIT.SW == RCC_SYS_SRC_HSI)
    {/*  clock source is HSI */
          sysClk = HSI_CLK;
    }
    else if(RCC->CR.BIT.SW == RCC_SYS_SRC_HSE)
    {/*  clock source is HSE */
          sysClk = HSE_CLK;
    }
    else if(RCC->CR.BIT.SW == RCC_SYS_SRC_PLL)
    {/*  clock source is PLL */
          if(RCC->CFGR.BIT.PLLSRC == PLL_SRC_HSI)  
          {
                sysClk = HSI_CLK*2;
          }
          else if(RCC->CFGR.BIT.PLLSRC == PLL_SRC_HSE)
          {
                sysClk = HSE_CLK;
          }
          else
          {
              assert_param(0);
          }
          sysClk /= (RCC->CFGR.BIT.PLLXTPRE);
          sysClk *= (RCC->CFGR.BIT.PLLF);
          sysClk /= (1<<RCC->CFGR.BIT.PLLK);
          sysClk /= 2;
    }
    else
    {
        assert_param(0);
    }
    
     Clocks->sysFreq = sysClk;
     sysClk >>= RCC->CFGR.BIT.HPRE;
     Clocks->hclkFreq = sysClk;
     Clocks->pclkFreq = sysClk >> RCC->CFGR.BIT.PPRE0;
}
  
/**
  * @brief    restore default clocks
  * @retval None
  */
void RCC_Reset(void)
{
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    /* switch to  HSI*/
    RCC->CR.BIT.SW = RCC_SYS_SRC_HSI;
    
    /* wait switch ready */
    while(RCC->CR.BIT.SW != RCC_SYS_SRC_HSI);
    
    /*  close HSE and PLL */
    RCC->CR.V32 = 0;
    
    /*  clear  CFGR  */
    RCC->CFGR.V32 =  0; 

    /*  clear interrupt enable bits */
    RCC->CIENR.V32 = 0;
    
    /* clear interrupt flags */
    RCC->CICLR.V32 = 0xFFFFFFFF;

    /* clear clock gates */
    RCC->AHBENR.V32[0] = 0;
    RCC->AHBENR.V32[1] = 0;
    RCC->AHBENR.V32[2] = 0;
    
    RCC->APB0ENR.V32[0] = 0;
    RCC->APB0ENR.V32[1] = 0;
    RCC->APB0ENR.V32[2] = 0;
    
    /* clear reset flags */
    RCC->RSTCLR.V32  = 0xFFFFFFFF;

    /* clear RC Trim */
    RCC->HSICAL.V32  = 0;
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief    configure HSE (external clock)
  * @param OnOffState   open or close the HSE
  *     @arg @b SH_ON    open HSE
  *     @arg @b SH_OFF   close HSE
  * @retval None
  */
void RCC_HSEOnOff(SH_CmdState OnOffState)
{
    assert_param(IS_CMD_STATE(OnOffState));
       
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CR.BIT.HSEON = OnOffState;

   /* Lock RCC registers */
    RCC_REGS_LOCK();
}

/**
  * @brief    wait for HSE ready
  * @param TimeOut   timeout ticks
  * @retval SH_ErrorStatus 
  *      @arg @b SH_SUCCESS  HSE is ready 
  *      @arg @b SH_ERROR     timeout
  */
SH_ErrorStatus RCC_WaitForHSEReady(uint32_t TimeOut)
{
    assert_param(TimeOut > 0);
    do{
        if(RCC->CR.BIT.HSERDY)
            break;
        TimeOut--;
    }while(TimeOut > 0);
    return (TimeOut==0?SH_ERROR:SH_SUCCESS);
}

/**
  * @code   Example : HSI increase one step
       RCC_AdjustHSITrimValue(RCC_HSITRIM_INC1);    
  * @endcode 
  *@brief   adjust HSI frequency
  *@param HSITrimValue  trim value
  *     @arg @b RCC_HSITRIM_INC0  ( + 0 step )
  *     @arg @b RCC_HSITRIM_INC1  ( + 1 step )
  *     @arg @b RCC_HSITRIM_INC2  ( + 2 step )
  *     @arg @b RCC_HSITRIM_INC3  ( + 3 step )
  *     @arg @b RCC_HSITRIM_DEC4  ( - 4 step )
  *     @arg @b RCC_HSITRIM_DEC3  ( - 3 step  )
  *     @arg @b RCC_HSITRIM_DEC2  ( - 2 step )
  *     @arg @b RCC_HSITRIM_DEC1  ( - 1 step )
  *@retval  Trim reference value
  */
uint32_t RCC_AdjustHSITrimValue(RCC_HSITRIM_Type HSITrimValue)
{
    assert_param(IS_HSITRIM(HSITrimValue));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    /* set trim value*/
    RCC->HSICAL.BIT.HSITRIM = HSITrimValue;
    
    /* start  */
    RCC->HSICAL.BIT.TRIMRUN = SH_SET;
    
    while(RCC->HSICAL.BIT.TRIMRUN == SH_SET);
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();    
	
    return RCC->HSICAL.BIT.TRIMREF;
}

/**
  * @brief  Fill the RCC_PLL_InitStruct member with its default value.
  * @param  PLLInit : pointer to a @ref RCC_PLL_InitTypeDef structure which will be initialized.
  * @retval None
  */
void RCC_PLLStructInit(RCC_PLL_InitTypeDef* PLLInit)
{
    /* Reset GPIO init structure parameters values */
    PLLInit->xpreDiv = PLL_XTPRE_DIV2;
    PLLInit->pllSRC  = PLL_SRC_HSI;
    PLLInit->pllMul  = 56;			
    PLLInit->pllDiv  = 2;		// (24/2)*56/2/2=168MHz
}

/**
  * @code   Example
       // config PLLHSI is 8MHz and PLL's output is 100MHz
       // 1. PLL source is HSI     ;can be PLL_SRC_HSI or PLL_SRC_HSE
       // 2. PLL pre-divider is 2  ;can be 1~7
       // 3. PLL F = 56            ;can be 1~255
       // 4. PLL K = 2             ;can be 1~32
       // 5. PLL output = HSI(12MHz)/pre-divider(2) * F(56)/K(2)/Fixed(2) = 168MHz
       RCC_PllInitTypeDef pllInit;
       pllInit.pllSRC    = PLL_SRC_HSI;  
       pllInit.xpreDiv   = PLL_XTPRE_DIV1; 
       pllInit.pllMul    = 25; 
       pllInit.pllDiv    = 1;  
  * @endcode 
  */
/**
  * @brief             configure PLL
  * @param PLLInit  PLL parameters @ref RCC_PLL_InitTypeDef
  * @retval None
  */
void RCC_PLLConfig(RCC_PLL_InitTypeDef* PLLInit)
{
    assert_param(IS_PLL_XTPRE(PLLInit->xpreDiv));
    assert_param(IS_PLL_SRC(PLLInit->pllSRC));
    assert_param(PLLInit->pllMul >= 31);
    assert_param((PLLInit->pllDiv >= 1 && PLLInit->pllDiv <= 32));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->CFGR.BIT.PLLXTPRE = PLLInit->xpreDiv;
    RCC->CFGR.BIT.PLLSRC   = PLLInit->pllSRC;
    RCC->CFGR.BIT.PLLF     = PLL_PLLF(PLLInit->pllMul);
    if(PLLInit->pllDiv == 1)
        RCC->CFGR.BIT.PLLK = 0;
    else if(PLLInit->pllDiv == 2)
        RCC->CFGR.BIT.PLLK = 1;
    else if(PLLInit->pllDiv == 4)
        RCC->CFGR.BIT.PLLK = 2;
    else if(PLLInit->pllDiv == 8)
        RCC->CFGR.BIT.PLLK = 3;
    else if(PLLInit->pllDiv == 16)
        RCC->CFGR.BIT.PLLK = 4;
    else if(PLLInit->pllDiv == 32)
        RCC->CFGR.BIT.PLLK = 5;
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();           
}

/**
  *@brief   open or close PLL
  *@param OnOffState  open or close PLL
  *     @arg @b SH_ON   open PLL
  *     @arg @b SH_OFF  close PLL
  *@retval None
  */
void RCC_PLLOnOff(SH_CmdState OnOffState)
{
    assert_param(IS_CMD_STATE(OnOffState));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    RCC->CR.BIT.PLLON = OnOffState;
    
   /* Lock RCC registers */
    RCC_REGS_LOCK();        
}

/**
  * @brief    wait for PLL ready
  * @param    TimeOut   timeout ticks
  * @retval   SH_ErrorStatus 
  *      @arg @b SH_SUCCESS  PLL is ready 
  *      @arg @b SH_ERROR     timeout
  */
SH_ErrorStatus RCC_WaitForPLLReady(uint32_t TimeOut)
{
    assert_param(TimeOut > 0);
    do{
        if(RCC->CR.BIT.PLLRDY)
            break;
        TimeOut--;
    }while(TimeOut > 0);
    return (TimeOut==0?SH_ERROR:SH_SUCCESS);
}

/**
  *@brief    configure system clock
  *@param SysClkSrc  assign system clock source
  *     @arg @b RCC_SYS_SRC_HSI   
  *     @arg @b RCC_SYS_SRC_HSE  
  *     @arg @b RCC_SYS_SRC_PLL   
  * @retval None
  */
void RCC_SysClkConfig(RCC_SysSource_Type SysClkSrc)
{    
    volatile uint8_t tmpSysClk;
    assert_param(IS_SYS_SOURCE(SysClkSrc));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CR.BIT.SW = SysClkSrc;
    __DSB();
    
    /* wait switching ready */
    do{
        tmpSysClk = RCC->CR.BIT.SWS;
    }
    while(RCC->CR.BIT.SW != tmpSysClk );
 
    /* Lock RCC registers */
    RCC_REGS_LOCK();            
}

/**
  * @brief    get current system clock source 
  * @retval RCC_SysSource_Type current system clock source 
  *     @arg @b RCC_SYS_SRC_HSI  
  *     @arg @b RCC_SYS_SRC_HSE 
  *     @arg @b RCC_SYS_SRC_PLL  
  */
RCC_SysSource_Type RCC_GetSysClkSource(void)
{
    return ((RCC_SysSource_Type)RCC->CR.BIT.SWS);
}

/**
  *@brief    configure HCLK
  *@param  HCLK_DIV  system clock's divider
  *     @arg @b  RCC_HCLK_DIV1   system clock / 1
  *     @arg @b  RCC_HCLK_DIV2   system clock / 2
  *     @arg @b  RCC_HCLK_DIV4   system clock / 4
  *     @arg @b  RCC_HCLK_DIV8   system clock / 8
  *@retval  None
  */
void RCC_HCLKConfig(RCC_HCLK_DIV_Type HCLK_DIV)
{
    assert_param(IS_HCLK_DIV(HCLK_DIV));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CFGR.BIT.HPRE = HCLK_DIV;
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();               
}

/**
  *@brief    configure PCLK
  *@param  PCLK_DIV  HCLK's divider
  *     @arg @b  RCC_PCLK_DIV1   HCLK / 1
  *     @arg @b  RCC_PCLK_DIV2   HCLK / 2
  *     @arg @b  RCC_PCLK_DIV4   HCLK / 4
  *     @arg @b  RCC_PCLK_DIV8   HCLK / 8
  *@retval  None
  */
void RCC_PCLKConfig(RCC_PCLK_DIV_Type PCLK_DIV)
{
    assert_param(IS_PCLK_DIV(PCLK_DIV));
   
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->CFGR.BIT.PPRE0 = PCLK_DIV;
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();               
}



/**
 * @brief    configure interrupt enable or disable
  *@param  INT_SRC  interrupt source: can be one or more.
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_INT_HSERDY: HSE Ready Interrupt Enable
  *     @arg @b  RCC_INT_PLLRDY: PLL Ready Interrupt Enable   
  *@param  NewState enable or disable interrupt
  *     @arg @b  SH_ENABLE  
  *     @arg @b  SH_DISABLE
  *@retval  None
   */
void RCC_INTConfig(uint32_t INT_SRC, SH_FunctionalState NewState)
{
    assert_param(IS_RCC_INTSRCS(INT_SRC));
    assert_param(IS_FUNCTION_STATE(NewState));
    
    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    if(NewState == SH_ENABLE)
        RCC->CIENR.V32 |= INT_SRC;
    else
        RCC->CIENR.V32 &= ~INT_SRC;      
   
    /* Lock RCC registers */
    RCC_REGS_LOCK();         

}


/**
 * @brief    Get RCC interrupt flag
  *@param  INTFlag  interrupt flag
  *   This parameter can be one of following values:
  *     @arg @b  RCC_INT_HSERDY:  HSE ready interrupt flag
  *     @arg @b  RCC_INT_PLLRDY   PLL ready interrupt flag
  *     @arg @b  RCC_INT_CSMHSE:  HSE clock abnormal flag
  *     @arg @b  RCC_INT_CSMPLL:  PLL clock abnormal flag
  *@retval  SH_FlagStatus RCC Interrupt status
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus  RCC_GetINTStatus(RCC_INT_Type INTFlag)
{
    assert_param(IS_RCC_INTFLAG(INTFlag));
    
    return ((SH_FlagStatus)((RCC->CISTR.V32 & INTFlag) ? SH_SET : SH_RESET));
}


/**
 * @brief  clear RCC interrupt flag
  *@param  INTFlag  interrupt source, can select more than one
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_INT_HSERDY   HSE ready interrupt flag
  *     @arg @b  RCC_INT_PLLRDY   PLL ready interrupt flag
  *     @arg @b  RCC_INT_CSMHSE   CSM  abnormal flag
  *     @arg @b  RCC_INT_CSMPLL   CSM  abnormal flag
  *@retval  None
   */
void  RCC_ClearINTStatus(uint32_t INTFlag)
{
    assert_param(IS_RCC_INTFLAGS(INTFlag));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();

    /* CSMC bit for CSMHSE & CSMPLL */
    if((INTFlag & RCC_INT_CSMHSE) == RCC_INT_CSMHSE)
        INTFlag |= RCC_INT_CSMPLL;        
    
    RCC->CICLR.V32 = INTFlag;    
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();            
}



/**
  * @code   Example : open ADC1 and ADC2 module's clock
        RCC_AHBPeriphClockOnOff(RCC_AHB_ADC1 | RCC_AHB_ADC2, SH_ON);
  * @endcode 
  */
/**
  * @brief     open or close AHB modules' clock gate
  * @param  AHBModules:  AHB module bits @ref RCC_AHB_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_AHB_GPIO: for GPIO Module: Includes Config and IO
  *     @arg @b  RCC_AHB_MCM0:  for MCM0 Module
  *     @arg @b  RCC_AHB_MCM1:  for MCM1 Module
  *     @arg @b  RCC_AHB_MACP: for MACP Module
  *     @arg @b  RCC_AHB_CRC:  for CRC Module
  *     @arg @b  RCC_AHB_PCA0: for PCA0 Module
  *     @arg @b  RCC_AHB_PCA1: for PCA1 Module
  *     @arg @b  RCC_AHB_PCA2: for PCA2 Module
  *     @arg @b  RCC_AHB_PCA3: for PCA3 Module
  *     @arg @b  RCC_AHB_DMA:  for DMA Module
  *     @arg @b  RCC_AHB_SYSCFG:  for SYSCFG Module
  *     @arg @b  RCC_AHB_SDFM0:  for SDFM0 Module
  *     @arg @b  RCC_AHB_ADC0:  for ADC0 Module
  *     @arg @b  RCC_AHB_ADC1:  for ADC1 Module
  *     @arg @b  RCC_AHB_ADC2:  for ADC2 Module
  *@param OnOffState  open or close related clock gate
  *     @arg @b  SH_ON  open related clock gate
  *     @arg @b  SH_OFF  close related clock gate
  * @retval   None
   */ 
void  RCC_AHBPeriphClockOnOff(uint32_t AHBModules,SH_CmdState OnOffState)
{
    assert_param(IS_AHB_MODULES(AHBModules));
    assert_param(IS_CMD_STATE(OnOffState));
    
    /* Unlock RCC registers */
	RCC_REGS_UNLOCK();
    
    if(RCC_AHB_ALL == AHBModules)
    {
        if(OnOffState == SH_ON)
        {
            RCC->AHBENR.V32[0]= 0xFFFFF;
            RCC->AHBENR.V32[1]= 0xFFFFF;
            RCC->AHBENR.V32[2]= 0xFFFFF;
        }
        else
        {
            RCC->AHBENR.V32[0]= 0;
            RCC->AHBENR.V32[1]= 0;
            RCC->AHBENR.V32[2]= 0;
        }        
    }
    else
    {
        switch(AHBModules)
        {
            case RCC_AHB_GPIO:
                RCC->AHBENR.BIT.GPIOEN = OnOffState;
                break;
            case RCC_AHB_MCM0:
                RCC->AHBENR.BIT.MCM0EN = OnOffState;
                break;
            case RCC_AHB_MCM1:
                RCC->AHBENR.BIT.MCM1EN = OnOffState;
                break;
            case RCC_AHB_MACP:
                RCC->AHBENR.BIT.MACPEN = OnOffState;
                break;
            case RCC_AHB_CRC:
                RCC->AHBENR.BIT.CRCEN = OnOffState;
                break;
            case RCC_AHB_PCA0:
                RCC->AHBENR.BIT.PCA0EN = OnOffState;
                break;
            case RCC_AHB_PCA1:
                RCC->AHBENR.BIT.PCA1EN = OnOffState;
                break;
            case RCC_AHB_PCA2:
                RCC->AHBENR.BIT.PCA2EN = OnOffState;
                break;
            case RCC_AHB_PCA3:
                RCC->AHBENR.BIT.PCA3EN = OnOffState;
                break;
            case RCC_AHB_DMA:
                RCC->AHBENR.BIT.DMAEN = OnOffState;
                break;
            case RCC_AHB_SYSCFG:
                RCC->AHBENR.BIT.SYSCFGEN = OnOffState;
                break;
            case RCC_AHB_SDFM0:
                RCC->AHBENR.BIT.SDFM0EN = OnOffState;
                break;
            case RCC_AHB_ADC0:
                RCC->AHBENR.BIT.ADC0EN = OnOffState;
                break;
            case RCC_AHB_ADC1:
                RCC->AHBENR.BIT.ADC1EN = OnOffState;
                break;
            case RCC_AHB_ADC2:
                RCC->AHBENR.BIT.ADC2EN = OnOffState;
                break;
        }
    }
           
    /* Lock RCC registers */
	RCC_REGS_LOCK();   
}

/*!  RCC APB0 Modules */
/**
  * @brief  open or close APB0 modules' clock gate
  * @param  APB0Modules:  APB0 module bits @ref RCC_APB0_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_APB0_TIM0
  *     @arg @b  RCC_APB0_TIM1
  *     @arg @b  RCC_APB0_QEI0
  *     @arg @b  RCC_APB0_QEI1
  *     @arg @b  RCC_APB0_UART0
  *     @arg @b  RCC_APB0_UART1
  *     @arg @b  RCC_APB0_UART2
  *     @arg @b  RCC_APB0_UART3
  *     @arg @b  RCC_APB0_SPI0
  *     @arg @b  RCC_APB0_SPI1
  *     @arg @b  RCC_APB0_TWI0
  *     @arg @b  RCC_APB0_WWDT
  *     @arg @b  RCC_APB0_EXTI
  *     @arg @b  RCC_APB0_AMOC
  *     @arg @b  RCC_APB0_CAN0
  *@param OnOffState  open or close related clock gate
  *     @arg @b  SH_ON  open related clock gate
  *     @arg @b  SH_OFF  close related clock gate
  * @retval  None
  */
void  RCC_APB0PeriphClockOnOff(uint32_t APB0Modules,SH_CmdState OnOffState)
{
    assert_param(IS_APB0_MODULES(APB0Modules));
    assert_param(IS_CMD_STATE(OnOffState));
    
    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();
    if(RCC_APB0_ALL == APB0Modules)
    {
        if(OnOffState == SH_ON)
        {
            RCC->APB0ENR.V32[0]= 0xFFFFF;
            RCC->APB0ENR.V32[1]= 0xFFFFF;
            RCC->APB0ENR.V32[2]= 0xFFFFF;
        }
        else
        {
            RCC->APB0ENR.V32[0]= 0;
            RCC->APB0ENR.V32[1]= 0;
            RCC->APB0ENR.V32[2]= 0;
        }        
    }
    else
    {    
        switch(APB0Modules)
        {
            case RCC_APB0_TIM0:
                RCC->APB0ENR.BIT.TIM0EN = OnOffState;
                break;
            case RCC_APB0_TIM1:
                RCC->APB0ENR.BIT.TIM1EN = OnOffState;
                break;
            case RCC_APB0_QEI0:
                RCC->APB0ENR.BIT.QEI0EN = OnOffState;
                break;
            case RCC_APB0_QEI1:
                RCC->APB0ENR.BIT.QEI1EN = OnOffState;
                break;
            case RCC_APB0_UART0:
                RCC->APB0ENR.BIT.UART0EN = OnOffState;
                break;
            case RCC_APB0_UART1:
                RCC->APB0ENR.BIT.UART1EN = OnOffState;
                break;
            case RCC_APB0_UART2:
                RCC->APB0ENR.BIT.UART2EN = OnOffState;
                break;
            case RCC_APB0_UART3:
                RCC->APB0ENR.BIT.UART3EN = OnOffState;
                break;
            case RCC_APB0_SPI0:
                RCC->APB0ENR.BIT.SPI0EN = OnOffState;
                break;
            case RCC_APB0_SPI1:
                RCC->APB0ENR.BIT.SPI1EN = OnOffState;
                break;
            case RCC_APB0_TWI0:
                RCC->APB0ENR.BIT.TWI0EN = OnOffState;
                break;
            case RCC_APB0_WWDT:
                RCC->APB0ENR.BIT.WWDTEN = OnOffState;
                break;
            case RCC_APB0_EXTI:
                RCC->APB0ENR.BIT.EXTIEN = OnOffState;
                break;
            case RCC_APB0_AMOC:
                RCC->APB0ENR.BIT.AMOCEN = OnOffState;
                break;
            case RCC_APB0_CAN0:
                RCC->APB0ENR.BIT.CAN0EN = OnOffState;
                break;
        }   
    }

   /* Lock RCC registers */
	RCC_REGS_LOCK();   
}

/**
  * @code   Example : reset CRC and MACP module
        RCC_AHBPeriphReset(RCC_AHB_CRC|RCC_AHB_MACP);
  * @endcode 
  */
/**
  * @brief     reset AHB modules
  * @param  AHBModules: AHB module bits @ref RCC_AHB_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_AHB_GPIO: for GPIO Module: Includes Config and IO
  *     @arg @b  RCC_AHB_MCM0:  for MCM0 Module
  *     @arg @b  RCC_AHB_MCM1:  for MCM1 Module
  *     @arg @b  RCC_AHB_MACP: for MACP Module
  *     @arg @b  RCC_AHB_CRC:  for CRC Module
  *     @arg @b  RCC_AHB_PCA0: for PCA0 Module
  *     @arg @b  RCC_AHB_PCA1: for PCA1 Module
  *     @arg @b  RCC_AHB_PCA2: for PCA2 Module
  *     @arg @b  RCC_AHB_PCA3: for PCA3 Module
  *     @arg @b  RCC_AHB_DMA:  for DMA Module
  *     @arg @b  RCC_AHB_SYSCFG:  for SYSCFG Module
  *     @arg @b  RCC_AHB_SDFM0:  for SDFM0 Module
  *     @arg @b  RCC_AHB_ADC0:  for ADC0 Module
  *     @arg @b  RCC_AHB_ADC1:  for ADC1 Module
  *     @arg @b  RCC_AHB_ADC2:  for ADC2 Module
  * @retval   None
   */ 
void RCC_AHBPeriphReset(uint32_t AHBModules)
{
    assert_param(IS_AHB_MODULES(AHBModules));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    switch(AHBModules)
    {
        case RCC_AHB_GPIO:
            RCC->AHBRSTR.BIT.GPIORST = 1;
            break;
        case RCC_AHB_MCM0:
            RCC->AHBRSTR.BIT.MCM0RST = 1;
            break;
        case RCC_AHB_MCM1:
            RCC->AHBRSTR.BIT.MCM1RST = 1;
            break;
        case RCC_AHB_MACP:
            RCC->AHBRSTR.BIT.MACPRST = 1;
            break;
        case RCC_AHB_CRC:
            RCC->AHBRSTR.BIT.CRCRST = 1;
            break;
        case RCC_AHB_PCA0:
            RCC->AHBRSTR.BIT.PCA0RST = 1;
            break;
        case RCC_AHB_PCA1:
            RCC->AHBRSTR.BIT.PCA1RST = 1;
            break;
        case RCC_AHB_PCA2:
            RCC->AHBRSTR.BIT.PCA2RST = 1;
            break;
        case RCC_AHB_PCA3:
            RCC->AHBRSTR.BIT.PCA3RST = 1;
            break;
        case RCC_AHB_DMA:
            RCC->AHBRSTR.BIT.DMARST = 1;
            break;
        case RCC_AHB_SYSCFG:
            RCC->AHBRSTR.BIT.SYSCFGRST = 1;
            break;
        case RCC_AHB_SDFM0:
            RCC->AHBRSTR.BIT.SDFM0RST = 1;
            break;
        case RCC_AHB_ADC0:
            RCC->AHBRSTR.BIT.ADC0RST = 1;
            break;
        case RCC_AHB_ADC1:
            RCC->AHBRSTR.BIT.ADC1RST = 1;
            break;
        case RCC_AHB_ADC2:
            RCC->AHBRSTR.BIT.ADC2RST = 1;
            break;
    }
    
    do{__NOP();}while(RCC->AHBRSTR.V32 != 0);
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();   
}


/**
  * @brief    reset APB0 modules
  * @param  APB0Modules:  APB0 module bits @ref RCC_APB0_Type
  *   This parameter can be any combination of following values:
  *     @arg @b  RCC_APB0_TIM0
  *     @arg @b  RCC_APB0_TIM1
  *     @arg @b  RCC_APB0_QEI0
  *     @arg @b  RCC_APB0_QEI1
  *     @arg @b  RCC_APB0_UART0
  *     @arg @b  RCC_APB0_UART1
  *     @arg @b  RCC_APB0_UART2
  *     @arg @b  RCC_APB0_UART3
  *     @arg @b  RCC_APB0_SPI0
  *     @arg @b  RCC_APB0_SPI1
  *     @arg @b  RCC_APB0_TWI0
  *     @arg @b  RCC_APB0_WWDT
  *     @arg @b  RCC_APB0_EXTI
  *     @arg @b  RCC_APB0_AMOC
  *     @arg @b  RCC_APB0_CAN0
  * @retval  None
  */
void RCC_APB0PeriphReset(uint32_t APB0Modules)
{
    assert_param(IS_APB0_MODULES(APB0Modules));

    /* Unlock RCC registers */
    RCC_REGS_UNLOCK();
    
    RCC->APB0RSTR.V32 = APB0Modules;
    switch(APB0Modules)
    {
        case RCC_APB0_TIM0:
            RCC->APB0RSTR.BIT.TIM0RST = 1;
            break;
        case RCC_APB0_TIM1:
            RCC->APB0RSTR.BIT.TIM1RST = 1;
            break;
        case RCC_APB0_QEI0:
            RCC->APB0RSTR.BIT.QEI0RST = 1;
            break;
        case RCC_APB0_QEI1:
            RCC->APB0RSTR.BIT.QEI1RST = 1;
            break;
        case RCC_APB0_UART0:
            RCC->APB0RSTR.BIT.UART0RST = 1;
            break;
        case RCC_APB0_UART1:
            RCC->APB0RSTR.BIT.UART1RST = 1;
            break;
        case RCC_APB0_UART2:
            RCC->APB0RSTR.BIT.UART2RST = 1;
            break;
        case RCC_APB0_UART3:
            RCC->APB0RSTR.BIT.UART3RST = 1;
            break;
        case RCC_APB0_SPI0:
            RCC->APB0RSTR.BIT.SPI0RST = 1;
            break;
        case RCC_APB0_SPI1:
            RCC->APB0RSTR.BIT.SPI1RST = 1;
            break;
        case RCC_APB0_TWI0:
            RCC->APB0RSTR.BIT.TWI0RST = 1;
            break;
        case RCC_APB0_WWDT:
            RCC->APB0RSTR.BIT.WWDTRST = 1;
            break;
        case RCC_APB0_EXTI:
            RCC->APB0RSTR.BIT.EXTIRST = 1;
            break;
        case RCC_APB0_AMOC:
            RCC->APB0RSTR.BIT.AMOCRST = 1;
            break;
        case RCC_APB0_CAN0:
            RCC->APB0RSTR.BIT.CAN0RST = 1;
            break;
    }   
    
    do{__NOP();}while(RCC->AHBRSTR.V32 != 0);
    
    /* Lock RCC registers */
    RCC_REGS_LOCK();       
}

/**
  * @brief     Get RCC reset flag
  * @param  ResetFlag:   RCC reset flag mask
  *     @arg @b  RCC_RST_PIN
  *     @arg @b  RCC_RST_LVR
  *     @arg @b  RCC_RST_POWERON
  *     @arg @b  RCC_RST_SOFTWARE
  *     @arg @b  RCC_RST_IWDT
  *     @arg @b  RCC_RST_WWDT
  * @retval  SH_FlagStatus RCC reset flag
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus RCC_GetResetFlag(RCC_RESET_Type ResetFlag)
{
    assert_param(IS_RCC_RESET_Type(ResetFlag));

    return ((RCC->RSTSTR.V32 & ResetFlag) ? SH_SET : SH_RESET);
}


/**
  * @brief  clear RCC reset flags
  * @param  ResetFlags:   RCC reset flag clear bits
  *     @arg @b  RCC_RST_PIN
  *     @arg @b  RCC_RST_LVR
  *     @arg @b  RCC_RST_POWERON
  *     @arg @b  RCC_RST_SOFTWARE
  *     @arg @b  RCC_RST_IWDT
  *     @arg @b  RCC_RST_WWDT
   * @retval None
  */
void RCC_ClearResetFlag(uint8_t ResetFlags)
{
    assert_param(IS_RCC_RESET_Types(ResetFlags));

    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();

    RCC->RSTCLR.V32 = ResetFlags;
        
   /* Lock RCC registers */
	RCC_REGS_LOCK();           
}


/**
  * @brief  Enable or Disable CSM function
  * @param  OnOffState: New CSM function status
  *     @arg @b SH_ON
  *     @arg @b SH_OFF
  * @retval None
  */
void RCC_CSMOnOff(SH_CmdState OnOffState)
{
    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();
    
    if(OnOffState == SH_ON)
        RCC->CR.BIT.CSMON = 1;
    else
        RCC->CR.BIT.CSMON = 0;

    /* Lock RCC registers */
	RCC_REGS_LOCK();           
}

/**
  * @brief  Get CSM flag status
  * @param  CSM_Flag: CSM source
  *     @arg @b  RCC_INT_CSMHSE
  *     @arg @b  RCC_INT_CSMPLL
  * @retval  SH_FlagStatus CSM flag status
  *     @arg @b  SH_SET    the flag is setted
  *     @arg @b  SH_RESET  the flag is cleared
  */
SH_FlagStatus RCC_CSMGetFlagStatus(uint32_t CSM_Flag)
{
    assert_param(CSM_Flag == RCC_INT_CSMHSE || CSM_Flag == RCC_INT_CSMPLL);
    
    if( RCC->CISTR.V32 & CSM_Flag)
        return SH_SET;
    else
        return SH_RESET;
    
}

/**
  * @brief  clear CSM flags
  * @retval None
  */
void RCC_CSMClearFlag(void)
{
    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();
    
    RCC->CICLR.BIT.CSMC = 1;
    
   /* Lock RCC registers */
	RCC_REGS_LOCK();           
    
}


/**
  * @brief  Set Clock Smooth Switch delay time
  * @param  dlymode: delay time mode
  * @retval None
  */
void RCC_ClockSmoothSwitch(RCC_CLKSS_Type dlymode)
{
    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();
    
    RCC->CLKSS.BIT.SSDLY = dlymode;
    
   /* Lock RCC registers */
	RCC_REGS_LOCK();           
}

/**
  * @brief  Set Clock Gate On delay time
  * @param  dlymode: delay time mode
  * @retval None
  */
void RCC_ClockOnDelay(RCC_CLKGON_Type dlymode)
{
    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();
    
    RCC->CLKSS.BIT.PYDLY = dlymode;
    
   /* Lock RCC registers */
	RCC_REGS_LOCK();               
}

/**
  * @brief  Tune on HSE
  * @retval HSE is ready or not
  *     @arg @b  SH_TRUE  HSE is ready
  *     @arg @b  SH_FALSE  HSE is not ready
  */
SH_Bool RCC_HSE_TuneOn(void)
{
    int timeout = 0;
    uint32_t csel;
    uint32_t rsel;
        
    /* Unlock RCC registers */
 	RCC_REGS_UNLOCK();
    
    for(csel = 0; csel < 4; csel++)
    {
        for(rsel = 0; rsel < 4; rsel++)
        {    
            RCC->HSEOPT.V32 = (csel<<4)|(rsel<<0);
            RCC->CR.BIT.HSEON = 1;    
            for(timeout = 0; timeout < 10000; timeout++)
            {
                if(RCC->CR.BIT.HSERDY)
                    break;
            }
            if(timeout < 10000)
                break;
        }
        if(timeout < 10000)
             break;
    }
    
   /* Lock RCC registers */
	RCC_REGS_LOCK();        
    
    return RCC->CR.BIT.HSERDY ? SH_TRUE : SH_FALSE;        
}



/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


