/**
  ******************************************************************************
  * @file    sh3xf2xx_sc0_syscfg.c
  * @author  sinowealth
  * @version V1.0.0
  * @date    07-March-2022 
  * @brief   This file provides firmware functions to manage the following 
  *         
  *  @verbatim
  *
  *          ===================================================================
  *                                   How to use this driver
  *          ===================================================================
  *
  *  @endverbatim
  *
  ******************************************************************************
  * @attention
  *
  * SINOWEALTH IS SUPPLYING THIS SOFTWARE FOR USE EXCLUSIVELY SH_ON SINOWEALTH'S 
  * MICROCONTROLLER PRODUCTS. IT IS PROTECTED UNDER APPLICABLE COPYRIGHT LAWS. 
  * THIS SOFTWARE IS FOR GUIDANCE IN ORDER TO SAVE TIME. AS A RESULT, SINOWEALTH 
  * SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES 
  * WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT OF SUCH FIRMWARE AND/OR
  * THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION CONTAINED HEREIN IN 
  * CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2022 Sinowealth</center></h2>
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "sh3xf2xx_sc0_syscfg.h"

/** @addtogroup SH3xF2xx_sc0_StdLib_Driver
  * @{
  */

/** @defgroup SYSCFG_MODULE SYSCFG 
  * @brief SYSCFG driver modules
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/ 
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/** @defgroup SYSCFG_Exported_Functions Public Functions
  * @{
  */ 
  
/**
  * @brief  Open or close LVR module.
  * @param  OnOffState: state of the BOD
  *          This parameter can one of the following values:
  *            @arg SH_ON   
  *            @arg SH_OFF  
  * @retval None
  */
void SYSCFG_LVROnOff(SH_CmdState OnOffState)
{
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));
    
    SYSCFG->PWRCR.BIT.LVREN = (uint32_t)OnOffState;
}  

/**
  * @brief  Config the specified LVR voltage.
  * @param  LVR_VoltageLevel: LVR voltage level
  *          This parameter can be one of the following values:
  *          @arg LVR_VoltageLevel_4V1  
  *          @arg LVR_VoltageLevel_2V8 
  * @retval None
  */
void SYSCFG_LVRVoltageLevelConfig(uint16_t LVR_VoltageLevel)
{ 
    /* Check the parameters */
    assert_param(IS_LVR_VOLTAGE_LEVEL(LVR_VoltageLevel));
    
    SYSCFG->PWRCR.V32 &= ~SYSCFG_PWRCR_VLVR_Msk;
    SYSCFG->PWRCR.V32 |= LVR_VoltageLevel;
}

  
/** @defgroup SYSCFG_Group1 BOD Configuration and function
 *  @brief    BOD Configuration and function
 *
@verbatim    
 ===============================================================================
                      BOD Configuration and function
 ===============================================================================  
   
@endverbatim
  * @{
  */
  
/**
  * @brief  Initialize BOD mode and voltage threshold.
  * @param  SYSCFG_BOD_Mode: Specifie the BOD mode.
  *          This parameter can be a value of @ref SYSCFGBODMode_Type.
  * @param  SYSCFG_BOD_Level: Specifie the BOD voltage threshold.
  *          This parameter can be a value of @ref SYSCFGBODLevel_Type.
  * @retval None
  */ 
void SYSCFG_BODInit(SYSCFGBODMode_Type SYSCFG_BOD_Mode, SYSCFGBODLevel_Type SYSCFG_BOD_Level)
{
    uint32_t temp;

    /* Check the parameters */
    assert_param(IS_SYSCFG_BOD_MODE(SYSCFG_BOD_Mode));
    assert_param(IS_SYSCFG_BOD_LEVEL(SYSCFG_BOD_Level));

    temp = SYSCFG->PWRCR.V32;
    temp &= ~(uint32_t)((3UL<<SYSCFG_PWRCR_BODMD_Pos) | (0xFUL<<SYSCFG_PWRCR_VBOD_Pos));        
    temp |= ((uint32_t)SYSCFG_BOD_Mode << SYSCFG_PWRCR_BODMD_Pos) | ((uint32_t)SYSCFG_BOD_Level<<SYSCFG_PWRCR_VBOD_Pos); 
    SYSCFG->PWRCR.V32 = temp;
}    

/**
  * @brief  Enable or disable the specified BOD interrupt.
  * @param  NewState: New state of BOD interrupt.
  *          This parameter can be one of the following values:
  *          @arg SH_ENABLE  
  *          @arg SH_DISABLE 
  * @retval None
  */
void SYSCFG_BODINTConfig(SH_FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTION_STATE(NewState));
    
    if (NewState != SH_DISABLE)
    {
        SYSCFG->PWRCR.V32 |= SYSCFG_PWRCR_BODIE_Msk;
    }
    else
    {
        SYSCFG->PWRCR.V32 &= ~SYSCFG_PWRCR_BODIE_Msk;
    }
}

/**
  * @brief  Open or close BOD module.
  * @param  OnOffState: state of the BOD
  *          This parameter can one of the following values:
  *            @arg SH_ON   
  *            @arg SH_OFF  
  * @retval None
  */
void SYSCFG_BODOnOff(SH_CmdState OnOffState)
{   
    /* Check the parameters */
    assert_param(IS_CMD_STATE(OnOffState));
    SYSCFG->PWRCR.BIT.BODEN = (uint32_t)OnOffState;
}


/**
  * @brief  Check whether the specified BOD flag is set or not.
  * @param  BOD_Flag: specifies the flag to check.
  *          This parameter can be one of the following values:
  *            @arg SYSCFG_BOD_FLAG_BODIF: BOD interrupt pending flag.
  *            @arg SYSCFG_BOD_FLAG_BODF: BOD output flag.
  * @retval New state of BOD Flag(SH_SET or SH_RESET).
  */
SH_FlagStatus SYSCFG_BODGetFlagStatus(uint8_t BOD_Flag)
{
    SH_FlagStatus bitStatus;

    /* Check the parameters */
    assert_param(IS_SYSCFG_BOD_FLAG(BOD_Flag));
    
    if ((SYSCFG->PWRSR.V32 & BOD_Flag) != SH_RESET)
    {
        bitStatus = SH_SET;
    }
    else
    {
        bitStatus = SH_RESET;
    }
    
    return bitStatus;
}

/**
  * @brief  Clear BOD interrupt pending bit.
  * @retval None
  */
void SYSCFG_BODClearFlag(void)
{
    /* Clear BODIF bit */
		SYSCFG->PWRSR.BIT.BODIF = 0;
}


  
/**
  * @}
  */


/** @defgroup SYSCFG_Group2 NMI trigger source configuration
 *  @brief    NMI trigger source configuration
 *
@verbatim    
 ===============================================================================
                      NMI trigger source configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */

/**
  * @brief  Enters SLEEP mode.
  * @param  SYSCFG_SLEEPEntry: specifies if SLEEP mode in entered with WFI or WFE instruction.
  *          This parameter can be one of the following values:
  *            @arg SYSCFG_SLEEPEntry_WFI: enter STOP mode with WFI instruction
  *            @arg SYSCFG_SLEEPEntry_WFE: enter STOP mode with WFE instruction
  * @retval None
  */  
void SYSCFG_EnterSLEEPMode(uint8_t SYSCFG_SLEEPEntry)
{
    /* Check the parameters */
    assert_param(IS_SYSCFG_SLEEP_ENTRY(SYSCFG_SLEEPEntry));
    
    /* Select SLEEP mode entry */
    if(SYSCFG_SLEEPEntry == SYSCFG_SLEEPEntry_WFI)
    {   
        /* Request Wait For Interrupt */
        __WFI();
    }
    else
    {
        /* Request Wait For Event */
        __WFE();
    }
}

/**
  * @brief  Enters STOP mode.
  *   
  * @note   In Stop mode, all I/O pins keep the same state as in Run mode.
  * @note   When exiting Stop mode by issuing an interrupt or a wakeup event, 
  *         the HSI RC oscillator is selected as system clock.
  * @param  SYSCFG_STOPEntry: specifies if STOP mode in entered with WFI or WFE instruction.
  *          This parameter can be one of the following values:
  *            @arg SYSCFG_STOPEntry_WFI: enter STOP mode with WFI instruction
  *            @arg SYSCFG_STOPEntry_WFE: enter STOP mode with WFE instruction
  * @retval None
  */
void SYSCFG_EnterSTOPMode(uint8_t SYSCFG_STOPEntry)
{
    /* Check the parameters */
    assert_param(IS_SYSCFG_STOP_ENTRY(SYSCFG_STOPEntry));
    
#if defined (__FPU_USED) && (__FPU_USED == 1U)
    //Disable FPU
     SCB->CPACR = 0;
    //power down FPU
	SCnSCB->CPPWR = ((3U << 10U*2U) | (3U << 11U*2U));
#endif    
    
    /* Set SLEEPDEEP bit of Cortex System Control Register */
    SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;

    /* Select STOP mode entry */
    if(SYSCFG_STOPEntry == SYSCFG_STOPEntry_WFI)
    {   
        /* Request Wait For Interrupt */
        __WFI();
    }
    else
    {
        /* Request Wait For Event */
        __WFE();
    }
    /* Reset SLEEPDEEP bit of Cortex System Control Register */
    SCB->SCR &= (uint32_t)~((uint32_t)SCB_SCR_SLEEPDEEP_Msk);  
    
#if defined (__FPU_USED) && (__FPU_USED == 1U)
    //power up  FPU
	SCnSCB->CPPWR = ((0U << 10U*2U) | (0U << 11U*2U));
    //Enable FPU 
    SCB->CPACR |= ((3U << 10U*2U) |           /* enable CP10 Full Access */
                   (3U << 11U*2U)  );         /* enable CP11 Full Access */
#endif        
}


/**
  * @brief  Configure NMI interrupt trigger source.
  * @param  TriggerSource: Specifie NMI interrupt trigger source.
  *          This parameter can be any combination of the following values:
  *            @arg SYSCFG_NMI_TRIGGER_CSM: CSM trigger NMI interrupt.
  *            @arg SYSCFG_NMI_TRIGGER_BOD: BOD trigger NMI interrupt.
  *            @arg SYSCFG_NMI_TRIGGER_EXTI0: EXTI0 trigger NMI interrupt.       
  * @param  NewState: New state of NMI source .
  *          This parameter can be one of the following values:
  *          @arg SH_ENABLE  
  *          @arg SH_DISABLE 
  * @retval None
  */
void SYSCFG_NMIInterruptTriggerConfig(uint16_t TriggerSource, SH_FunctionalState NewState)
{
    uint32_t temp = 0;
    
    /* Check the parameters */
    assert_param(IS_SYSCFG_NMI_TRIGGER_SOURCE(TriggerSource));
    assert_param(IS_FUNCTION_STATE(NewState));

    temp = SYSCFG->SAFR.V32 & (uint32_t)0x0000FFFF;

    if (NewState != SH_DISABLE)
    {
        temp |= ((uint32_t)0x5AA5UL << 16) | TriggerSource;
        SYSCFG->SAFR.V32 = temp;
    }
    else
    {
        temp &= ~TriggerSource;
        temp |= ((uint32_t)0x5AA5UL << 16);
        SYSCFG->SAFR.V32 = temp;
    }        
}



/**
  * @}
  */


/** @defgroup SYSCFG_Group3 Special Pin configuration
 *  @brief    Special Pin configuration
 *
@verbatim    
 ===============================================================================
                      Special Pin configuration
 ===============================================================================  
   
@endverbatim
  * @{
  */


/**
  * @brief  Specifie SWJ pin alternate function.
  * @param  SWJPin_Mode: specifie the value for SWJ pin mode.
  *          This parameter can be one of the following values:
  *            @arg SYSCFG_EMUIO_GPIO_ALL
  *            @arg SYSCFG_EMUIO_PA10_PA11  
  *            @arg SYSCFG_EMUIO_PA10_PA12  
  * @retval None
  */
void SYSCFG_SWJPinConfig(uint8_t SWJPin_Mode)
{
    uint32_t temp = 0;

    /* Check the parameters */
    assert_param(IS_SYSCFG_SWJ_PIN_MODE(SWJPin_Mode));

    temp = SYSCFG->SAFR.V32 & 0xFFFF;
    temp &= ~((uint32_t)SYSCFG_EMUIO_GPIO_ALL);
    temp |= (uint32_t)SWJPin_Mode;   
    SYSCFG->SAFR.V32 = ((uint32_t)0x5AA5 << 16) | temp;
}

/**
  * @brief  Specifie XTAL pin alternate
  * @param  OSCPin_Mode: specifie the value for XTAL pin mode.
  *          This parameter can be one of the @ref SYSCFG_OSCPin_Type enum values:
  *            @arg SYSCFG_OSC_GPIO
  *            @arg SYSCFG_OSC_Crystal
  *            @arg SYSCFG_OSC_ExtClock
  * @retval None
  */
void SYSCFG_OSCPinConfig(SYSCFG_OSCPin_Type OSCPin_Mode)
{
    uint32_t temp = 0;

    /* Check the parameter */
    assert_param(IS_SYSCFG_OSC_PIN_MODE(OSCPin_Mode));

    temp = SYSCFG->SAFR.V32 & (~SYSCFG_SAFR_OSCCFG_Msk);
    
    temp |= (uint32_t)OSCPin_Mode;   
    SYSCFG->SAFR.V32 = ((uint32_t)0x5AA5 << 16) | temp;  
}


/**
  * @brief  Lock or unlock CRAM to allow modification
  * @param  blkMask: CRAM block to locked or unlocked.
  * @param  cmdState: lock or unlock
  *          @arg SH_ON  locked
  *          @arg SH_OFF unlocked
  * @retval None
  */
void SYSCFG_CRAMLock(uint32_t blkMask, SH_CmdState cmdState)
{
    if(cmdState == SH_ON)
    {
        SYSCFG->CRAMLOCK.V32 = 0X5AA50000 | (blkMask);
    }
    else
    {
        uint32_t mask = SYSCFG->CRAMLOCK.V32;
        mask &= ~(blkMask);
        SYSCFG->CRAMLOCK.V32 = 0X5AA50000 | (mask);        
    }
}

/**
  * @}
  */








/**
  * @}
  */

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT Sinowealth *****END OF FILE****/


